from django.db import models
from django.core.exceptions import ObjectDoesNotExist
import logging

log = logging.getLogger('django')

# Priority settings for ToDoItem
HIGH = 'High'
MEDIUM = 'Medium'
LOW = 'Low'
ITEM_PRIORITY_CHOICES = (
	(HIGH, HIGH),
	(MEDIUM, MEDIUM),
	(LOW, LOW),
)
# Status settings for ToDoItem
OPEN = 'Open'
IN_PROGRESS = 'In Progress'
ON_HOLD = 'On Hold'
CLOSED = 'Closed'
ITEM_STATUS_CHOICES = (
	(OPEN, OPEN),
	(IN_PROGRESS, IN_PROGRESS),
	(ON_HOLD, ON_HOLD),
	(CLOSED, CLOSED),
)

'''Stores system settings as key-value pairs
'''
class SystemSetting(models.Model):

	DEFAULT_SETTINGS = {
		'work_filter': 			'true',
		'personal_filter': 		'true',
		'closed_filter': 		'true',
		'on_hold_filter': 		'true',
		'in_progress_filter': 	'true',
		'open_filter': 			'true',
		'priority_color_high': 	'ff2600',
		'priority_color_medium':'ffe15b',
		'priority_color_low': 	'ffffff',
		'update_old_days': 		'10',
		'update_aged_days': 	'3',
		'update_purge_days':	'10',
		'update_color_purge': 	'808080',
		'update_color_old': 	'ff2600',
		'update_color_aged': 	'ffe066',
		'update_color_new': 	'ffffff',
		'status_color_closed': 	'86d786',
		'status_color_onhold': 	'ff9e9f',
		'status_color_inprogress':'9dedfa',
		'status_color_open': 	'ffffff',
		'purge_days': 			'90'
	}

	key = models.CharField(max_length = 30, unique = True)
	value = models.CharField(max_length = 50)

	def __str__(self):
		return (self.key + ': ' + self.value)

	@staticmethod
	def set_defaults():
		# assign defaults if they don't exist
		for key, value in SystemSetting.DEFAULT_SETTINGS.items():
			try:
				SystemSetting.objects.get(key=key)
			except ObjectDoesNotExist:
				SystemSetting.objects.create(key=key, value=value)
				log.info('Creating default setting: ' + key + ' = ' + value)

'''The item itself and data refering to the item as a whole
'''
class ToDoItem(models.Model):
	title = models.CharField(max_length = 100, default = 'New Item')
	customer = models.CharField(max_length = 50, blank = True)
	contact = models.CharField(max_length = 50, blank = True)
	priority = models.CharField(max_length = 10, default = LOW, choices = ITEM_PRIORITY_CHOICES)
	datetime_created = models.DateTimeField(auto_now_add = True)
	status = models.CharField(max_length = 10, default = OPEN, choices = ITEM_STATUS_CHOICES)

	def get_latest_update(self):
		return self.itemupdate_set.latest('datetime_created')

	def __str__(self):
		return self.title

'''Individual updates on an item. Each update is meant to convey a singe action taken.
'''
class ItemUpdate(models.Model):
	item = models.ForeignKey('ToDoItem', on_delete = models.CASCADE)
	datetime_created = models.DateTimeField(auto_now_add = True)
	text = models.TextField(blank = False)

	def __str__(self):
		return str(self.item) + '-' + str(self.datetime_created)