import logging
from datetime import datetime
from unittest.mock import patch
from django.shortcuts import render
from django.core.urlresolvers import reverse
from django.http import HttpRequest, HttpResponse, HttpResponseRedirect, HttpResponseServerError, HttpResponseNotAllowed
from django.test import TestCase, Client

from .apps import TodoCXConfig
from .models import SystemSetting, ToDoItem, ItemUpdate, HIGH, LOW, OPEN, CLOSED
from .views import create_item, delete_item, live_edit, main_page, settings, edit_item, create_update, purge_records

# disable logging so we don't clutter our test results
logging.disable(logging.CRITICAL)

# use this with patch decorator to extract context from a rendered response
def get_context(call_args):
	args, kwargs = call_args
	request, template, context = args
	return context
	

##########################################	
##########################################
#  MODEL TESTS
##########################################
##########################################

# SystemSetting model tests
##########################################
class SystemsSettingTest(TestCase):

	def setUp(self):
		self.setting1 = SystemSetting.objects.create(key = 'key1', value = 'value1')
		self.setting2 = SystemSetting(key = 'key2', value = 'value2')

	def tearDown(self):
		pass

	def test_string_rep(self):
		self.assertEqual(str(self.setting1), 'key1: value1')
		self.assertEqual(str(self.setting2), 'key2: value2')

	def test_default_settings(self):
		setting3 = SystemSetting.objects.create(key = 'purge_days', value = '90')
		SystemSetting.set_defaults()
		setting_exists = SystemSetting.objects.get(key='open_filter')
		setting_size = len(SystemSetting.objects.filter(key='purge_days'))
		self.assertTrue(setting_exists)
		self.assertEqual(setting_size, 1)



# ToDoItem model tests
##########################################
class ToDoItemTest(TestCase):

	def setUp(self):
		self.item1 = ToDoItem.objects.create(title = 'title1')
		self.item2 = ToDoItem.objects.create(title = 'title2', customer = 'cust2', contact = 'contact2', priority = HIGH, status = CLOSED)

	def tearDown(self):
		pass

	def test_string_rep(self):
		self.assertEqual(str(self.item1), 'title1')
		self.assertEqual(str(self.item2), 'title2')

	def test_get_latest_update(self):
		update1 = ItemUpdate.objects.create(item = self.item1, text = 'update1')
		update2 = ItemUpdate.objects.create(item = self.item1, text = 'update2')
		new_update = self.item1.get_latest_update()
		self.assertEquals(new_update.text, 'update2')


# ItemUpdate model tests
##########################################
class ItemUpdateTest(TestCase):

	def setUp(self):
		self.item1 = ToDoItem.objects.create(title = 'title1')
		self.item2 = ToDoItem.objects.create(title = 'title2')
		self.update1 = ItemUpdate.objects.create(item = self.item1, text = 'update1')
		self.update2 = ItemUpdate.objects.create(item = self.item2, text = 'update2')

	def tearDown(self):
		pass

	def test_string_rep(self):
		current_datetime = datetime.today()
		self.update1.datetime_created = current_datetime
		self.update2.datetime_created = current_datetime
		self.assertEqual(str(self.update1), 'title1-' + str(current_datetime))
		self.assertEqual(str(self.update2), 'title2-' + str(current_datetime))



##########################################	
##########################################
#  VIEW TESTS
##########################################
##########################################


# Main Page view tests
##########################################
class MainPageTest(TestCase):

	def setUp(self):
		self.item1 = ToDoItem.objects.create(title = 'title1')
		self.item2 = ToDoItem.objects.create(title = 'title2')
		ItemUpdate.objects.create(item=self.item1, text='text')
		ItemUpdate.objects.create(item=self.item2, text='text')
		SystemSetting.objects.create(key='closed_filter', value='val1')

	def tearDown(self):
		pass

	@patch('todocx.views.render')
	def test_main_page(self, mock_render):
		SystemSetting.objects.create(key='purge_days', value='7')
		request = HttpRequest()
		response = main_page(request)
		context = get_context(mock_render.call_args)
		item_list = context['item_list']
		filter_list = context['filters']
		self.assertTrue(len(item_list) > 0)
		self.assertTrue(len(filter_list) > 0)

	@patch('todocx.views.render')
	def test_purge_records(self, mock_render):
		SystemSetting.objects.create(key='purge_days', value='0')
		request = HttpRequest()
		response = main_page(request)
		context = get_context(mock_render.call_args)
		item_list = context['item_list']
		filter_list = context['filters']
		self.assertFalse(len(item_list) > 0)
		self.assertTrue(len(filter_list) > 0)

	def test_create_item(self):
		request = HttpRequest()
		response = create_item(request)
		self.assertEqual(response.status_code, 302)


# Edit Item view tests
##########################################
class EditItemPageTest(TestCase):

	def setUp(self):
		self.item1 = ToDoItem.objects.create(title = 'title1')

	def tearDown(self):
		pass

	@patch('todocx.views.render')
	def test_edit_item_page(self, mock_render):
		request = HttpRequest()
		response = edit_item(request, self.item1.id)
		context = get_context(mock_render.call_args)
		item_exists = 'item' in context.keys()
		self.assertTrue(item_exists)

	def test_delete_item(self):
		request = HttpRequest()
		response = delete_item(request, self.item1.id)
		self.assertEqual(str(response), str(HttpResponseRedirect(reverse('todocx:main_page'))))

	def test_create_update_non_post(self):
		# a non-POST request is denied with a 405 error
		request = HttpRequest()
		request.method = 'GET'
		response = create_update(request)
		self.assertEqual(str(response), str(HttpResponseNotAllowed(['POST'])))

	def test_create_update_invalid_post(self):
		# an invalid POST request is denied with a 500 error
		request = HttpRequest()
		request.method = 'POST'
		request.POST = {'total_farts': 50}
		response = create_update(request)
		self.assertEqual(str(response), str(HttpResponseServerError(reason = "POST request is invalid.")))

	def test_create_update_valid_entry(self):
		# fully valid request and successful save
		request = HttpRequest()
		request.method = 'POST'
		request.POST = {'item_id':self.item1.id, 'text': 'text1'}
		response = create_update(request)
		self.assertEqual(str(response), str(HttpResponse(reverse('todocx:edit_item', kwargs = {'item_id': self.item1.id}))))
		item3 = ToDoItem.objects.get(pk = self.item1.id)	
		update = item3.get_latest_update()
		self.assertEquals(update.text, 'text1')



# SystemSetting view tests
##########################################
class SettingsPageTest(TestCase):

	def setUp(self):
		SystemSetting.objects.create(key='key1', value='val1')
		SystemSetting.objects.create(key='key2', value='val2')

	def tearDown(self):
		pass

	@patch('todocx.views.render')
	def test_settings_page(self, mock_render):
		request = HttpRequest()
		response = settings(request)
		context = get_context(mock_render.call_args)
		settings_list = context['settings_list']
		self.assertEquals(len(settings_list), 2)


# LiveEdit view tests
##########################################
class LiveEditTest(TestCase):

	def setUp(self):
		self.item2 = ToDoItem.objects.create(title = 'title2', customer = 'cust2', contact = 'contact2', priority = HIGH, status = CLOSED)
		self.modelid = self.item2.id

	def tearDown(self):
		pass

	def assertFields(self, item3):
		self.assertEqual(item3.title, 'title2')
		self.assertEqual(item3.customer, 'cust2')
		self.assertEqual(item3.contact, 'contact2')
		self.assertEqual(item3.priority, HIGH)
		self.assertEqual(item3.status, CLOSED)

	def test_non_post(self):
		# a non-POST request is denied with a 405 error
		request = HttpRequest()
		request.method = 'GET'
		response = live_edit(request)
		self.assertEqual(str(response), str(HttpResponseNotAllowed(['POST'])))
		item3 = ToDoItem.objects.get(pk = self.modelid)
		self.assertFields(item3)

	def test_invalid_post(self):
		# an invalid POST request is denied with a 500 error
		request = HttpRequest()
		request.method = 'POST'
		request.POST = {'total_farts': 50}
		response = live_edit(request)
		self.assertEqual(str(response), str(HttpResponseServerError(reason = "POST request is invalid.")))
		item3 = ToDoItem.objects.get(pk = self.modelid)
		self.assertFields(item3)

	def test_invalid_model(self):
		# an invalid model is denied with a 500 error
		request = HttpRequest()
		request.method = 'POST'
		request.POST = {'modelid': self.modelid, 'modelname': 'SmellyFart', 'fieldname': 'title', 'value': 'new_title'}
		response = live_edit(request)
		self.assertEqual(str(response), str(HttpResponseServerError(reason = "Model is not valid.")))
		item3 = ToDoItem.objects.get(pk = self.modelid)
		self.assertFields(item3)

	def test_invalid_field(self):
		# an invalid field is denied with a 500 error
		request = HttpRequest()
		request.method = 'POST'
		request.POST = {'modelid': self.modelid, 'modelname': 'ToDoItem', 'fieldname': 'smellyfart', 'value': 'really stinky'}
		response = live_edit(request)
		self.assertEqual(str(response), str(HttpResponseServerError(reason = "Field is not valid.")))
		item3 = ToDoItem.objects.get(pk = self.modelid)
		self.assertFields(item3)

	def test_bad_save(self):
		# a failed save is denied with a 500 error
		request = HttpRequest()
		request.method = 'POST'
		request.POST = {'modelid': 23578, 'modelname': 'ToDoItem', 'fieldname': 'title', 'value': 'new_title'}
		response = live_edit(request)
		self.assertEqual(str(response), str(HttpResponseServerError(reason = "Model could not be saved.")))
		item3 = ToDoItem.objects.get(pk = self.modelid)
		self.assertFields(item3)

	def test_valid_entry(self):
		# fully valid request and successful save
		request = HttpRequest()
		request.method = 'POST'
		request.POST = {'modelid': self.modelid, 'modelname': 'ToDoItem', 'fieldname': 'title', 'value': 'new_title'}
		response = live_edit(request)
		self.assertEqual(str(response), str(HttpResponse("SUCCESS")))
		item3 = ToDoItem.objects.get(pk = self.modelid)	
		self.assertEqual(item3.title, 'new_title')
		self.assertEqual(item3.customer, 'cust2')
		self.assertEqual(item3.contact, 'contact2')
		self.assertEqual(item3.priority, HIGH)
		self.assertEqual(item3.status, CLOSED)

	def test_new_update(self):
		# request to create a new ItemUpdate object
		request = HttpRequest()
		request.method = 'POST'
		request.POST = {'modelid': self.modelid,
						'modelname': 'ItemUpdate',
						'fieldname': 'text',
						'value': 'update text',
						'req_vals': '{"item": ' + str(self.modelid) + '}'}
		response = live_edit(request)
		self.assertEqual(str(response), str(HttpResponse("SUCCESS")))
		item3 = ToDoItem.objects.get(pk = self.modelid)	
		update = item3.get_latest_update()
		self.assertEquals(update.text, 'update text')




