import json

from django.shortcuts import render
from django.core.urlresolvers import reverse
from django.http import HttpResponse, HttpResponseRedirect, HttpResponseServerError, HttpResponseNotAllowed
from django.apps import apps

from todocx.models import SystemSetting, ToDoItem, ItemUpdate

import logging
from datetime import datetime, timedelta
from django.utils import timezone

# Get an instance of a logger
log = logging.getLogger('django')

# LiveEdit framework - Server side
##########################################
def live_edit(request):
	'''
	Receives a POST request with the following parameters:
		
		modelid: the database id of the model
		modelname: the name of the model object
		fieldname: the field of the object to edit
		value: the value to store in the field

	The framework sends SUCCESS unless there is an error, which results in an HTTP 500 (server error).
	If request is not a POST, the request will receive an HTTP 405 (response not allowed).
		'''

	if request.method != 'POST':
		log.exception('LiveEdit request is not a POST request')
		return HttpResponseNotAllowed(['POST'])
	# parse the request
	try:
		modelid = request.POST['modelid']
		modelname = request.POST['modelname']
		fieldname = request.POST['fieldname']
		value = request.POST['value']
	except:
		log.exception('LiveEdit POST is invalid')
		return HttpResponseServerError(reason = "POST request is invalid.")
	# find the model object manager
	try:
		model = apps.get_model('todocx', modelname)
	except:
		log.exception('LiveEdit model is invalid')
		return HttpResponseServerError(reason = "Model is not valid.")
	# find the field
	try:
		field = model._meta.get_field(fieldname)
	except:
		log.exception('LiveEdit field is invalid')
		return HttpResponseServerError(reason = "Field is not valid.")

	try:
		# if we've marked this as a new update
		# need to create a new ItemUpdate object and populate it
		if 'req_vals' in request.POST.keys():
			required_values = json.loads(request.POST['req_vals'])
			item = ToDoItem.objects.get(pk=required_values['item'])
			text = request.POST['value']
			if text == '':  # pragma: no cover
				raise ValueError('ItemUpdate.text cannot be an empty string')
			the_model = model.objects.create(item = item, text = text)
		else:
			the_model = model.objects.get(pk = modelid)
		# update the field
		setattr(the_model, fieldname, value)
		# save the model
		the_model.save()
	except:
		log.exception('LiveEdit model save failed')
		return HttpResponseServerError(reason = "Model could not be saved.")
	# return SUCCESS
	log.info('Successful LiveEdit request')
	return HttpResponse('SUCCESS')


# Create/Delete ToDoItem request
##########################################
def create_item(request):
	try:
		new_item = ToDoItem()
		new_item.save()
		return HttpResponseRedirect(reverse('todocx:edit_item', args=(new_item.id,)))
	except:  # pragma: no cover
		log.exception('ToDoItem creation failed')
		return HttpResponseRedirect(reverse('todocx:main_page'))

def delete_item(request, item_id):
	try:
		item_to_delete = ToDoItem.objects.get(pk=item_id)
		item_to_delete.delete()
	except:  # pragma: no cover
		log.exception('Failed to delete ToDoItem')
	return HttpResponseRedirect(reverse('todocx:main_page'))


# Main Page
##########################################
def main_page(request):
	# run the purge before displaying results
	purge_records()

	context = {}
	settings_list = {}
	try:
		context['item_list'] = ToDoItem.objects.all().order_by('datetime_created')
		filters = SystemSetting.objects.filter(key__contains = 'filter')
		context['filters'] = {}
		for filter in filters:
			context['filters'][filter.key] = {'value': filter.value, 'id': filter.id}
		for setting in SystemSetting.objects.all():
			settings_list[setting.key] = {'id': setting.id, 'value': setting.value}
		context['settings_list'] = settings_list
		aged_days = SystemSetting.objects.get(key='update_aged_days')
		aged_date = timezone.now() - timedelta(days = int(aged_days.value) )
		context['aged_date'] = aged_date
		old_days = SystemSetting.objects.get(key='update_old_days')
		old_date = timezone.now() - timedelta(days = int(old_days.value) )
		context['old_date'] = old_date
		# add coloring for items about to be purged
		# mark items that are within 10 days of the purge
		purge_days = int(SystemSetting.objects.get(key='purge_days').value) - int(SystemSetting.objects.get(key='update_purge_days').value);
		if purge_days < 0:
			purge_days = int(SystemSetting.objects.get(key='update_purge_days').value);
		purge_date = timezone.now() - timedelta(days = purge_days )
		context['purge_date'] = purge_date
	except:  # pragma: no cover
		log.exception('Main Page failed to load')
		raise
	return render(request, 'todocx/index.html', context)

# System Settings Page
##########################################
def settings(request):
	context = {}
	settings_list = {}
	try:
		for setting in SystemSetting.objects.all():
			settings_list[setting.key] = {'id': setting.id, 'value': setting.value}
		context['settings_list'] = settings_list
	except:  # pragma: no cover
		log.exception('System Settings page failed to load')
		raise
	return render(request, 'todocx/settings.html', context)

# View/Edit Item Page
##########################################
def edit_item(request, item_id):
	context = {}
	settings_list = {}
	try:
		context['item'] = ToDoItem.objects.get(pk=item_id)
		for setting in SystemSetting.objects.all():
			settings_list[setting.key] = {'id': setting.id, 'value': setting.value}
		context['settings_list'] = settings_list
		aged_days = SystemSetting.objects.get(key='update_aged_days')
		aged_date = timezone.now() - timedelta(days = int(aged_days.value) )
		context['aged_date'] = aged_date
		old_days = SystemSetting.objects.get(key='update_old_days')
		old_date = timezone.now() - timedelta(days = int(old_days.value) )
		context['old_date'] = old_date
	except:  # pragma: no cover
		log.exception('Edit Item failed to load')
		raise
	return render(request, 'todocx/edititem.html', context)

# Create Update call
##########################################
def create_update(request):
	if request.method != 'POST':
		log.exception('create_update() request is not a POST request')
		return HttpResponseNotAllowed(['POST'])
	# parse the request
	try:
		item_id = request.POST['item_id']
		text = request.POST['text']
	except:
		log.exception('create_update() POST is invalid')
		return HttpResponseServerError(reason = "POST request is invalid.")
	item = ToDoItem.objects.get(pk=item_id)
	ItemUpdate.objects.create(item=item, text=text)
	log.info('Successful create update request')
	return HttpResponse(reverse('todocx:edit_item', kwargs = {'item_id': item_id}))


# Purge job
##########################################
def purge_records():
	# set number of deleted records to 0
	del_recs = 0
	# calculate purge date
	purge_days = SystemSetting.objects.get(key='purge_days')
	purge_threshold = timezone.now() - timedelta(days = int(purge_days.value) )

	records = ToDoItem.objects.all()
	for record in records:
		if len(record.itemupdate_set.all()) <= 0:
			record.delete()
			del_recs += 1
		elif record.get_latest_update().datetime_created < purge_threshold:
			record.delete()
			del_recs += 1
	log.info(str(del_recs) + ' records purged')

